"use strict";

///
/// 使い方：
///   let r = new MartinRouter();
///   r.mount('/orders'     , funcOrders);
///   r.mount('/orders/123' , funcOrderDetail);
///   console.log(r.route('/orders'));
///                  //=> funcOrders
///   console.log(r.route('/orders/123'));
///                  //=> funcOrderDetail
///
class MartinRouter {

  constructor() {
    this._entries = [];   // パスと関数の組の配列
  }

  /// パスに対応した関数を登録する
  mount(path, func) {
    let x = {
      path: path
    , func: func
    };
    this._entries.push(x);
    return this;
  }

  /// パスに対応した関数を返す（なければnullを返す）
  route(path) {
    for (let x of this._entries) {
      if (path === x.path) {
        return x.func;                   // 関数を返す
      }
    }
    return null;   // どれにもマッチしなければnullを返す
  }

}

if (typeof module !== 'undefined') {
  module.exports = MartinRouter;
}
