"use strict";

(function() {

  /// ユーティリティ
  let U = {
    init: function() {
    }
    /// 新しいIDを生成する
  , newId: function() {
      return Math.floor(Math.random() * 100000);
    }
  };

  /// モデル（コアとなるデータとロジック）
  let M = (function() {
    function init() {
    }
    /// タスク
    class Task {
      constructor(id, title, done) {
        this.id    = id;
        this.title = title;
        this.done  = !! done;
      }
    }
    //
    return {init: init, Task: Task};
  })();

  /// コントローラ（イベントハンドラ）
  let C = {
    init: function() {
      C.registerHandlers();           // イベントハンドラを登録
    }
    /// イベントハンドラを登録する
  , registerHandlers: function() {
      $('#add-task').on('submit', C.doAddTask);
    }
    /// タスクを追加する
  , doAddTask: function() {
      /// thisは<form>なので、その子孫要素からテキストフィールドを探す
      let $input = $(this).find('input[type=text][name=title]');
      /// テキストフィールドが空でなければ、タスクを追加
      let title = $input.val().trim();
      if (title) {
        let task = new M.Task(U.newId(), title);
        V.addTask(task);
      }
      $input.val("");  /// テキストフィールドを空にする
      return false;    /// フォームデータを送信しない
    }
  };

  /// ビュー（表示に関する機能）
  let V = {
    init: function() {
    }
    /// タスクのDOM要素（<li>タグ）を作る
  , renderTask: function(task) {
      let id  = 'task-' + task.id;
      let cls = task.done ? 'task completed' : 'task';
      let arr =
        ['li', {'id': id, 'class': cls, 'data-id': task.id}
        , ['div', {'class': 'view'},
          , ['input', {'class': 'toggle', type: 'checkbox'}]
          , ['label', task.title]
          , ['button', {'class': 'destroy'}]
          ]
        , ['form'
          , ['input.edit', {type: 'text', value: task.title}]
          ]
        ];
      return a2dom(arr);
    }
    /// タスクリストに新しいタスクを追加する（<li>タグ）
  , addTask: function(task) {
      $('#todo-list').prepend(V.renderTask(task));
    }
  };

  /// 初期設定用
  function init() {
    U.init();
    M.init();
    V.init();
    C.init();
  }

  /// 初期設定を行う
  if (typeof $ !== 'undefined') { $(init); }

})();

if (typeof module !== 'undefined') { module.exports = TodoApp; }
